# File: trianglengin/trianglengin/utils/types.py
# Placeholder for Phase 1
"""
Shared type definitions for the Triangle Engine.
"""

from typing import TypeAlias

# Action representation (integer index)
ActionType: TypeAlias = int
